﻿-- Priority List 3
-- by Jadya
-- EU-Well of Eternity

local addonName, addonTable = ...
local L = addonTable.L
local C = addonTable.C
local addon = PriorityList

local AddColor = addonTable.AddColor

local chars = addon.CharFunctions
local items = addon.ItemFunctions

local dialogs = addon.Dialogs
addon.opt = {}

function chars:New(name, refresh)
 if not PLWish_List[name] then
  PLWish_List[name] = {}
  for diff = 1,3 do
   PLWish_List[name][diff] = {}
   PLWish_List[name][diff].lists  = { {},{},{} }
   PLWish_List[name][diff].looted = {}
  end
  addon:EventPulse("select_char", name)
  if refresh then
   addon:Refresh()
  end
 end
end

function chars:Rename(name, newname, refresh)
 local renamechar = function()
                     local c = PLWish_List[name]
					 PLWish_List[newname] = c
					 PLWish_List[name]    = nil
					 addon:EventPulse("select_char", newname, false)

                     if refresh then
					  addon:Refresh()
                     end
                    end

 if PLWish_List[name] and not PLWish_List[newname] then
  dialogs:Confirm(renamechar)
 end
end

function chars:Delete(name, refresh)

 local deletechar = function()
                     PLWish_List[name] = nil
					 addon:EventPulse("select_char", nil, false)
                     if refresh then
                      addon:Refresh()
                     end
                    end
 if PLWish_List[name] then
  if refresh then 
   dialogs:Confirm(deletechar)
  else
   deletechar()
  end
 end
end

function chars:AddItem(name, itemID, list, difficulty, refresh, position)
 if not name or name == "" then return end
 
 if not PLWish_List[name] then chars:New(name) end

 itemID = addon:Item2Token(itemID)
 
 for l = 1,3 do
  if tContains(PLWish_List[name][difficulty].lists[l], itemID) then return end
 end

 if position then
  table.insert(PLWish_List[name][difficulty].lists[list], position, itemID)
 else
  table.insert(PLWish_List[name][difficulty].lists[list], itemID)
 end
 
 if refresh then addon:Refresh() end
end

function chars:DeleteItem(name, itemID, list, difficulty, refresh)
 local t = chars:GetList(name, list, difficulty)
 if not t then return end
 
 itemID = addon:Item2Token(itemID)
 
 local index
 for i = 1,#t do
  if t[i] == itemID then
   index = i
   break
  end
 end

 if index then
  table.remove(t, index)
  --chars:UnsetLooted(name, itemID, difficulty)
  if refresh then addon:Refresh() end
 end
end

function chars:GetList(name, list, difficulty)
 if not PLWish_List[name] then return end

 return PLWish_List[name][difficulty].lists[list]
end

function chars:ClearLists(name, difficulty)
 if not PLWish_List[name] then return end
 
 for l = 1,3 do
  wipe(PLWish_List[name][difficulty].lists[l])
 end
 
 wipe(PLWish_List[name][difficulty].looted)
end

function chars:FindItem(name, itemID, difficulty)
 if not PLWish_List[name] then return end

 for list = 1,3 do
  local t = chars:GetList(name, list, difficulty)
  for i = 1,#t do
   if t[i] == itemID then
    return list, difficulty, i, PLWish_List[name][difficulty].looted[itemID]
   end
  end
 end
 return nil, difficulty
end

function chars:FindItemLink(name, link)
 local itemID = items:ExtractItemID(link)
 
 local list, difficulty, index = chars:FindItem(name, itemID, items:GetDifficulty(link))
 return list, difficulty, index
end

function chars:GetLootedItem(name, difficulty, itemID)
 return PLWish_List[name] and PLWish_List[name][difficulty].looted[itemID]
end

function chars:UnsetLooted(name, itemID, difficulty, refresh)
 PLWish_List[name][difficulty].looted[itemID] = nil
 
 if refresh then
  addon:Refresh()
 end
end

function chars:SetLooted(name, link, refresh, personal)

 local list, difficulty, index = chars:FindItemLink(name, link)
 -- no char or item not in their wish list

 local t = PLWish_List[name][difficulty]
 
 if not t or not list then return end

 local itemID = items:ExtractItemID(link)

 local olditem = difficulty and chars:GetLootedItem(name, difficulty, itemID)
 local result = true


 if olditem then
  local old_difficulty = items:GetDifficulty(olditem.link)
  local new_difficulty = items:GetDifficulty(link)
  
  if new_difficulty == old_difficulty then
   local new_score = items:GetBonusScore(items:ExtractBonuses(link))
   local old_score = items:GetBonusScore(items:ExtractBonuses(olditem.link))

   result = new_score > old_score
  else
   result = new_difficulty > old_difficulty
  end
  
  -- debug
  if result then print("..it's an upgrade!") end
 end
 
 if not result then
  return
 end
 
 if olditem then
  olditem.link = link 
 else
  t.looted[itemID] = { ["link"] = link }
 end
 t.looted[itemID].time = time()
 
 if addon:IsSuperUser() then
  print(L["addon_name"].." - "..name.." loots "..link.." (personal = "..tostring(personal)..")")
 end

 if personal then
  t.looted[itemID].personal = true
  -- set status for autosync
 else
  --chars:GetList(name, list, difficulty)[index] = -math.abs(chars:GetList(name, list, difficulty)[index])
 end

 if refresh then
  addon:Refresh()
 end
end