-- Priority List
-- by Fluffies- EU-Well of Eternity

local addonName, addonTable = ...
local L = addonTable.L
local AddColor = addonTable.AddColor


local tinsert, tremove, strsub, min, strbyte, strchar = table.insert, table.remove, string.sub, math.min, string.byte, string.char

local MSG_PREFIX       = "PriorityList"
local MSG_LIST_REQUEST = "getlist"
local MSG_LIST_PREPARE = "prepare"
local MSG_LIST_ACCEPT  = "acceptl"
local MSG_LIST_DATA    = "sndlist"
local MSG_SU_QUERY     = "suquery"
local MSG_SU_YES       = "suyes__"
local MSG_SU_GIVE      = "sugive_"
local MSG_SU_REMOVE    = "suremov"
local MSG_MESSAGE      = "message"

RegisterAddonMessagePrefix(MSG_PREFIX)

local list_separator = "_"

local addon = PriorityList
local comm = addon.CommFunctions
local dialogs = addon.Dialogs
local serializer = addon.Serializer

local pending_import = { curr = 0, max = 0 }
local pending_exports = {}

function comm:stringtobyte(s)
 local result = ""
 for i = 1,#s do
  local b = tostring(strbyte(s, i))
  while #b < 3 do
   b = "0"..b
  end
  result = result..b
 end
 return result
end

function comm:bytetostring(s)
 local i, result = 1, ""
 while i < #s do
  local c = strsub(s, i, i + 2)
  result = result..strchar(c)
  i = i + 3
 end
 return result
end

local export_timer
local last_time = 0
local export_curr
local function export_timerUpdate(self)
 local now = GetTime()

 if now < last_time + 0.2 then return end

 local index
 
 for k = 1,#pending_exports do
  if pending_exports[k].ready then
   index = k
  end
 end
 
 if not index then 
  comm:ExportTimer_Stop()
  return
 end

 if pending_exports[index][1] then
  comm:SendMessage(pending_exports[index].sender, MSG_LIST_DATA..pending_exports[index][1])
  tremove(pending_exports[index], 1)
 else
  tremove(pending_exports, index)
 end
 
 last_time = now
end

function comm:ExportTimer_Start()
 if not export_timer then
  export_timer = CreateFrame("Frame")
  export_timer:SetScript("OnUpdate", export_timerUpdate)
 end
 export_timer:Show()
end

function comm:ExportTimer_Stop()
 export_timer:Hide()
end

function comm:SendMessage(name, msg)
 SendAddonMessage(MSG_PREFIX, msg, "WHISPER", name)
end

-- SuperUser functions
function comm.SetSuperUser(name)
 comm:SendMessage(name, MSG_SU_GIVE..addon.guildname)
end

function comm.RemoveSuperUser(name)
 comm:SendMessage(name, MSG_SU_REMOVE..addon.guildname)
end

function comm:FindSuperUsers()
 SendAddonMessage(MSG_PREFIX, MSG_SU_QUERY..addon.guildname, "GUILD")
end
--

local function cut(s, data, chunksize)
 while #s > 0 do
  local i = min(#s, chunksize)
  tinsert(data, strsub(s, 1, i))
  s = strsub(s, i + 1, #s)
 end
end

addon.eventhandler:RegisterEvent("CHAT_MSG_ADDON")
function addon.eventhandler.CHAT_MSG_ADDON(prefix, message, channel, sender)
 local length = #MSG_LIST_REQUEST
 local pref = strsub(message, 1, length)
 local newmsg = strsub(message, length + 1, #message)
 
 if pref == MSG_LIST_REQUEST then
  -- 1) host gets an import request
  local _, name, _, guildname = strsplit(":", message)
  
  if guildname ~= addon.guildname then
   comm:SendMessage(sender, MSG_MESSAGE..L["request_wrong_guild"])
   return
  end
  
  local s_list, s_looted = "", ""
  
  local data = {}
  data.sender = sender
  
  if name == "all" then
   local s = ""
   for k in pairs(PLWish_List) do
    s = s..serializer:packLists(k)..serializer:packLooted(k)
   end

   cut(s, data, 245)

   tinsert(pending_exports, data)
   comm:SendMessage(sender, MSG_LIST_PREPARE..#data)
  elseif PLWish_List[name] then
   local s = serializer:packLists(name)..serializer:packLooted(name)
   cut(s, data, 245)
   tinsert(pending_exports, data)
   comm:SendMessage(sender, MSG_LIST_PREPARE..#data)
  else
   --print(AddColor(L["addon_name"], "LIGHT_BLUE").." - "..L["err_export_nochar"]) 
  end
 elseif pref == MSG_LIST_PREPARE then
  -- 2) client gets ready to import
  if pending_import.max > 0 then
   print(AddColor(L["addon_name"], "LIGHT_BLUE").." - "..L["import_busy"])
   return
  end

  local max = tonumber(newmsg)

  pending_import.data  = ""
  pending_import.max   = max
  pending_import.curr  = 0
  pending_import.ready = true
  pending_import.sender = sender
  dialogs:Load_Init(max, string.format(L["importing_from"], sender), function()
    pending_import.curr = 0 
    pending_import.max = -1 -- import canceled
	comm:SendMessage(pending_import.sender or ("<"..UNKNOWN..">"), MSG_MESSAGE..string.format(L["import_cancelled"], addon.playername))
   end)

  comm:SendMessage(sender, MSG_LIST_ACCEPT)
 elseif pref == MSG_LIST_ACCEPT then
  -- 3) host has green light to export
  for k,v in pairs(pending_exports) do
   if v.sender == sender then
    v.ready = true
    comm:ExportTimer_Start()
	print(AddColor(L["addon_name"], "LIGHT_BLUE").." - "..string.format(L["exporting"], sender))
	return
   end
  end
 elseif pref == MSG_LIST_DATA then
  -- 4) client receives data
  local t = pending_import
  if t.max == -1 or not pending_import.ready then -- import canceled
   return
  end
  pending_import.data = pending_import.data..newmsg
  t.curr = t.curr + 1
  dialogs:Load_SetProgress(t.curr)
  if t.curr >= t.max then
   comm:SendMessage(sender, MSG_MESSAGE..string.format(L["list_received"], addon.playername))
   serializer:unpack(sender, pending_import.data)
   pending_import.curr = 0
   pending_import.max  = 0
   pending_import.data = ""
   pending_import.ready = false
   dialogs:Load_Finalize()
   addon:EventPulse("import_list", sender)
  end
 -- superuser comm
 elseif pref == MSG_SU_QUERY then
  if addon:IsSuperUser() and newmsg == addon.guildname then
   comm:SendMessage(sender, MSG_SU_YES)
  end
 elseif pref == MSG_SU_YES then
  addon:SURadar_Add(sender) -- f_options.lua
 elseif pref == MSG_SU_GIVE and newmsg == addon.guildname then
  if not addon:IsSuperUser() then
   PLCharOptions["superuser"] = true
   comm:SendMessage(sender, MSG_MESSAGE..string.format(L["SU_given"], addon.playername))
   print(AddColor(L["addon_name"], "LIGHT_BLUE").." - "..string.format(L["SU_given_print"], sender))
  else
   comm:SendMessage(sender, MSG_MESSAGE..string.format(L["SU_notchanged"], addon.playername))
  end
 elseif pref == MSG_SU_REMOVE and newmsg == addon.guildname then
  if addon:IsSuperUser() then
   PLCharOptions["superuser"] = false
   comm:SendMessage(sender, MSG_MESSAGE..string.format(L["SU_removed"], addon.playername))
   print(AddColor(L["addon_name"], "LIGHT_BLUE").." - "..string.format(L["SU_removed_print"], sender))
  else
   comm:SendMessage(sender, MSG_MESSAGE..string.format(L["SU_notchanged"], addon.playername))
  end
 elseif pref == MSG_MESSAGE then
  print(AddColor(L["addon_name"], "LIGHT_BLUE").." - "..newmsg)
 end
end

function comm.Export(args)
 local name1, name2 = strsplit(" ", args)
 
 if not name1 or name1 == "" then return end
 
 if not name2 then
  name2 = addon:IsSuperUser() and "all" or addon.playername
 end
 local link = "[PriorityList - "..addon.playername.." ("..name2..":0)]"
 SendChatMessage(link ,"WHISPER" , nil, name1)
end

function comm:SetHyperlink(tooltip, link)

 if not addon:IsSuperUser() then return end
 
 local _, _, decode = strfind(link,  "prioritylist:([^%s]+)")
 local sender, name, flag1 = strsplit(":", comm:bytetostring(decode))
 if not sender or not name or not flag1 then
  -- bad version
  return
 end
 comm:SendMessage(sender, MSG_LIST_REQUEST..":"..name..":"..flag1..":"..addon.guildname)
end
function comm:LinkToChat(name)
 if not name or name == "" then return end

 local found
 for _ in pairs(PLWish_List) do
  found = true
  break
 end
 
 if not PLWish_List[name] and not found then return end
 
 local s = "[PriorityList - "..addon.playername.." ("..name..":0)]"
 if not DEFAULT_CHAT_FRAME.editBox:IsVisible() then
  DEFAULT_CHAT_FRAME.editBox:Show()
 end
 DEFAULT_CHAT_FRAME.editBox:SetFocus()
 DEFAULT_CHAT_FRAME.editBox:SetText(DEFAULT_CHAT_FRAME.editBox:GetText()..s)
end

-- handle priority list hyperlinks
local function chatFilter(self, event, msg, ...)

 -- hyperlink
 local pattern = "%[PriorityList %- ([^%s]+) %(([^%s]+)%)%]"
 local s = msg
 local result = ""
 local i, j, sender, flags = strfind(s, pattern)
 while i do
  if sender then
   -- we convert everything to ascii values to avoid addons
   -- that emphasize the player's name to interfere with the hyperlink
   local encode  = comm:stringtobyte(sender..":"..flags)
   result = result..strsub(s, 1, i - 1).."|cffff8000|Hprioritylist:"..encode.."|h[PriorityList - "..sender.."]|h|r"..strsub(s, j + 1, #s)
  end
  s = strsub(s, j + 1, #s)
  i, j, sender = strfind(s, pattern)
 end  
 
 if result ~= "" then
  return false, result, ...
 end
end

ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", chatFilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_GUILD", chatFilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_OFFICER", chatFilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY", chatFilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY_LEADER", chatFilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID", chatFilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_LEADER", chatFilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY", chatFilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", chatFilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", chatFilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER", chatFilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER_INFORM", chatFilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_CONVERSATION", chatFilter)

SLASH_PRIORITYLISTEXPORT1 = "/plexport"
SlashCmdList["PRIORITYLISTEXPORT"] = comm.Export